package de.ozgcloud.apilib.file.grpc;

import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataResponse;
import io.grpc.stub.StreamObserver;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PROTECTED)
class OzgCloudFileDownloadStreamObserver implements StreamObserver<GrpcGetBinaryFileDataResponse> {

	private final CompletableFuture<Boolean> streamFuture;
	private final OutputStream out;

	@Override
	public void onNext(GrpcGetBinaryFileDataResponse response) {
		writeToStream(response.getFileContent().toByteArray());
	}

	void writeToStream(byte[] contentPart) {
		try {
			out.write(contentPart);
		} catch (IOException e) {
			throw new TechnicalException("Download file error writing on OutputStream", e);
		}
	}

	@Override
	public void onError(Throwable t) {
		streamFuture.completeExceptionally(t);
	}

	@Override
	public void onCompleted() {
		streamFuture.complete(true);
	}
}
